//Module needed for hashing the password
const bcrypt = require('bcrypt');
const { json } = require('body-parser');
const { name } = require('ejs');

const multer = require('multer');
const upload = multer({storage: multer.memoryStorage()});

//Module needed for the validator
const { check, validationResult } = require('express-validator');


module.exports = function(app, shopData) {
//redirects users back to login page if they are not logged in
    const redirectLogin = (req, res, next) => {
        if (!req.session.userId ) {
        res.redirect('./login')
        } else { next (); }
        } 

    // Handle our routes
    app.get('/',function(req,res){
        res.render('index.ejs', shopData)
    });
    app.get('/about',function(req,res){
        res.render('about.ejs', shopData);
    });
    //const redirectLogin is added as a parameter the following pages that require users to be logged in.
    app.get('/search',redirectLogin,function(req,res){
        res.render("search.ejs", shopData);
    });

    app.get('/login',function(req,res){
        res.render('login.ejs', shopData)
    });

    app.get('/checkout',function(req,res){
        res.render('checkout.ejs', shopData)
    });


    app.get('/itemlist',redirectLogin,function(req,res){
        res.render('itemlist.ejs', shopData)
    });

    app.get('/listingdone',function(req,res){
        res.render('/listingdone.ejs', shopData)
    });

    app.get('/done',function(req,res){
        res.render('./done.ejs', shopData)
    });


    app.get('/cart',function(req,res){

    let sqlquery = "SELECT * FROM items"; // query database to get all the books
    // execute sql query
    db.query(sqlquery,(err, result) => {
        if (err) {
            res.redirect('./'); 
        }

        req.session.cart = [];

        let newData = Object.assign({}, shopData, {listingAdded: result });
        res.render("cart.ejs",newData);
     });
       
    });

    app.get('/',function(req,res){

        let sqlquery = "SELECT * FROM items"; // query database to get all the books
        // execute sql query
        db.query(sqlquery,(err, result) => {
            if (err) {
                res.redirect('./'); 
            }
            let newData = Object.assign({}, shopData, {listingAdded: result});
            res.render("itemlisted.ejs",newData)
         });

    });







    app.get('/search-result',[check('name').isEmpty()],function (req, res) {


        const errors = validationResult(req); 
        if (!errors.isEmpty()) {
           res.redirect('./search'); }
        else {
            let sqlquery = "SELECT * FROM items WHERE name LIKE '%" + req.query.keyword + "%'"; // query database to get all the books
            // execute sql query
            db.query(sqlquery, (err, result) => {
                if (err) {
                    res.redirect('./'); 
                }
                let newData = Object.assign({}, shopData, {listingAdded:result});
                res.render("itemlisted.ejs", newData)
            }); 
        }

    });

    app.post('/photoDB',upload.single("ProductImage"), (req,res) => {


        product_id = req.body.product_id;
        title = req.body.title;
        product = req.body.name;
        price = req.body.price;
        info = req.body.info;
        condition = req.body.condition;
        username = req.session.userId;
        image =req.file.buffer.toString("base64");
        req.session.cart = [];

        let sqlquery ="INSERT INTO items(title,name,price,info,condi,img,username) VALUES(?,?,?,?,?,?,?)";

        let newListing = [req.sanitize(title),req.sanitize(product),
                            req.sanitize(price),req.sanitize(info),
                            req.sanitize(condition),image,username,product_id]

        db.query(sqlquery, newListing,(err, result) => {
            if (err){
                return console.error(err.message);
            }
            else{

                let newData = Object.assign({}, shopData, {listingAdded: newListing, cart: req.session.cart});
                res.render("listingdone.ejs", newData)
            }
        }); 

    });


    app.get('/itemlisted',function(req,res){

        let sqlquery = "SELECT * FROM items"; // query database to get all the books
        // execute sql query
        db.query(sqlquery,(err, result) => {
            if (err) {
                res.redirect('./'); 
            }
            let newData = Object.assign({}, shopData, {listingAdded: result});
            res.render("itemlisted.ejs",newData)
         });

    });

    

    app.post('/add-to-cart',function(req,res){

        req.session.cart = [];

        let cart = req.session.cart;
     

        let item = {
            id: req.body.product_id,
            name: req.body.product_name,
            price: req.body.product_price
        };
        
        cart.push(item);
        console.log(cart);

        let newData = Object.assign({}, shopData, {cart: cart});
        res.render("cart",newData);
        
    });


    app.post('/check-out',function(req,res){
        res.redirect("checkout");
    });

    app.post('/final',function(req,res){
        res.render("done.ejs");
    });

    app.get('/register', function (req,res) {
        res.render('register', shopData);                                                                     
    }); 

// validation for email to check if it includes and @
// validation for first name last name and username to make sure its not empty 
// validation for pass to make sure its 8 characters at the minimum and 50 characters at the max 
    app.post('/registered',[check('email').isEmail()], 
                           [check('password').isLength({min: 8, max: 50})],
                           [check('first').notEmpty()],
                           [check('last').notEmpty()],
                           [check('username').notEmpty()],async (req,res) =>{
                            
        //validates a form input as an email address.
        const errors = validationResult(req);
        if (!errors.isEmpty()) {
            res.redirect('./register');
        }
        else {
            // saving data in database
            let sqlquery = "INSERT INTO userdata (username,firstname,lastname,email,hashedPassword) VALUES (?,?,?,?,?)";

            const bcrypt = require('bcrypt');
            const saltRounds = 10;
            const plainPassword = req.body.password; 
            const hash =  await bcrypt.hash(plainPassword,saltRounds);
            //sanitising each field using req.sanitize

            bcrypt.hash(plainPassword, saltRounds, function(err, hashedPassword) {
                let data = [req.sanitize(req.body.username),
                    req.sanitize(req.body.first),
                    req.sanitize(req.body.last),
                    req.sanitize(req.body.email), hashedPassword];


                // Store hashed password in your database.
                db.query(sqlquery, data, (err, result) => {
                    if (err) {
                        return console.error(err.message);
                    }
                    else{
                        let newData = Object.assign({}, shopData, {newUser:data});
                        res.render('registered.ejs',newData)

                    }
                    });
            })
            
}});

   
    app.get('/login',function(req,res){
        // Saves user session, given log in is successful
        req.session.userId = req.sanitize(req.body.username); 
        res.render("login.ejs", shopData);
    });

// Validation to make sure username is not empty and makes sure the password is at least 8 characters long.
    app.post('/loggedin',[check('username').notEmpty()],
                         [check('password').isLength({min: 8, max: 50})], function(req,res){
        const errors = validationResult(req);
        if (!errors.isEmpty()) {
            res.redirect('./login'); }
        else {
            let sqlQuery = "SELECT * FROM userdata WHERE username='"+req.sanitize(req.body.username)+"'";
            req.session.userId = req.sanitize(req.body.username);
        
        
            db.query(sqlQuery, function (err, result) {
                let hashedpassword = result[0].hashedPassword;
                console.log('result', result[0]);
                if(err) {
                    res.redirect("./");
                }
                
                
                bcrypt.compare(req.body.password,hashedpassword, function(err,result){
                    if(err){
                        res.redirect(err);
                    }
                    else if(result === true){
                        res.redirect("./correct");
                    }
                    else{
                        res.redirect("./incorrect");
                    }
                
                });
            });

        }

});
    app.get('/correct',function(req,res){
        res.render("correct.ejs", shopData);
    });
    app.get('/incorrect',function(req,res){
        res.render("incorrect.ejs", shopData);
    });

    app.get('/loggedout',function(req,res){
        res.render("loggedout.ejs", shopData);
    });

    //Page that is redirected to when user is logged in and wants to log out.
    app.get('/logout', redirectLogin, (req,res) => {
        req.session.destroy(err => {
        if (err) {
        return res.redirect('./')
        }
        res.redirect("./loggedout");
        })
    })


}
